// ImageMixerView.cpp : implementation of the CImageMixerView class
//

#include "stdafx.h"
#include "ImageMixer.h"

#include "ImageMixerDoc.h"
#include "ImageMixerView.h"

#include "Image.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView

IMPLEMENT_DYNCREATE(CImageMixerView, CScrollView)

BEGIN_MESSAGE_MAP(CImageMixerView, CScrollView)
	//{{AFX_MSG_MAP(CImageMixerView)
	ON_COMMAND(ID_FILE_OPEN_BASE, OnFileOpenBase)
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_INSERT_IMAGE, OnInsertImage)
	ON_WM_LBUTTONDOWN()
	ON_WM_NCHITTEST()
	ON_COMMAND(ID_FILE_SAVE, OnFileSave)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CScrollView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CScrollView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView construction/destruction

CImageMixerView::CImageMixerView():m_nWidth(1024),m_nHeight(740),
                                   m_pBaseImage(NULL),m_pSelectedShape(NULL)
{
	m_SettingsDlg.SetParent(this);

	m_pBackBrush = ::new SolidBrush(Color(255, 0, 0, 0));

	m_pMemBitmap   = ::new Bitmap(m_nWidth,m_nHeight,PixelFormat32bppPARGB);
	m_pMemGraphics = ::new Graphics(m_pMemBitmap);
}

CImageMixerView::~CImageMixerView()
{
	if(m_pBaseImage)
	{
		delete m_pBaseImage;
		m_pBaseImage = NULL;
	}

	delete m_pMemBitmap;
	delete m_pMemGraphics;
}

BOOL CImageMixerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView drawing

void CImageMixerView::OnDraw(CDC* pDC)
{
	CImageMixerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CRect ClipRect;
	pDC->GetClipBox(&ClipRect);

	//Just copy MemDC to real DC
	Graphics g(pDC->m_hDC);
	g.DrawImage(m_pMemBitmap,ClipRect.left,ClipRect.top,ClipRect.left,ClipRect.top,
				ClipRect.Width(),ClipRect.Height(),UnitPixel);
}

void CImageMixerView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();

	m_pMemGraphics->FillRectangle(m_pBackBrush,0,0,m_nWidth,m_nHeight);

	m_SettingsDlg.Create(IDD_SETTINGS,this);
	CRect Rect;
	m_SettingsDlg.GetClientRect(&Rect);
	m_SettingsDlg.ShowWindow(SW_SHOW);



	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = m_nWidth;
	sizeTotal.cy = m_nHeight;
	SetScrollSizes(MM_TEXT, sizeTotal);
}

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView printing

BOOL CImageMixerView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CImageMixerView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CImageMixerView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView diagnostics

#ifdef _DEBUG
void CImageMixerView::AssertValid() const
{
	CScrollView::AssertValid();
} 

void CImageMixerView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CImageMixerDoc* CImageMixerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CImageMixerDoc)));
	return (CImageMixerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CImageMixerView message handlers

void CImageMixerView::OnFileOpenBase() 
{
	static TCHAR BASED_CODE szFilter[] = _T("Image Files (*.bmp;*.jpg;*.gif;*.png;*.tiff)|*.bmp;*.jpg;*.gif;*.png;*.tiff|All Files (*.*)|*.*||");
 
	CFileDialog FileOpenDlg(TRUE,_T("png"),NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
								szFilter,NULL);
	if(IDOK == FileOpenDlg.DoModal())
	{
		CString strFileName = FileOpenDlg.GetPathName();
		if(!strFileName.IsEmpty())
		{
			this->LoadImage(strFileName);
		}
	}	
}

bool CImageMixerView::LoadImage(CString &strFileName)
{
	bool bRet = false;

	do
	{
		//Check for valid image extensions
		if(!IsValidImageFile(strFileName))
		{
			AfxMessageBox(_T("Unsupported Image type!!!"));
			break;
		}

		if(m_pBaseImage)
		{
			delete m_pBaseImage;
			m_pBaseImage = NULL;
		}

		//Load the image to MemDC
		m_pBaseImage = ::new Image(strFileName,FALSE);

		//Calculate the new image size
		if(m_pBaseImage)
		{
			RectF rect;
			Unit ut;
			m_pBaseImage->GetBounds(&rect,&ut);
			m_nWidth  = (int)rect.Width;
			m_nHeight = (int)rect.Height;

			
		}

		RefreshMemDC();
		
		//Adjust the scrollbar based on new size
		AdjustScroll();

		
		//Force screen updation
		Invalidate();


	}while(0);

	return bRet;

	
}

void CImageMixerView::RefreshMemDC()
{
	UpdateMemDC(m_pBaseImage);

	DrawAllShapes();

}

void CImageMixerView::UpdateMemDC(Image *pImg)
{
	if(pImg)
	{
		//Resize MemBitmap to hold the new image
		delete m_pMemBitmap;
		delete m_pMemGraphics;

		m_pMemBitmap   = ::new Bitmap(m_nWidth,m_nHeight,PixelFormat32bppPARGB);
		m_pMemGraphics = ::new Graphics(m_pMemBitmap);

		//Draw the loaded image to MemDC
		m_pMemGraphics->FillRectangle(m_pBackBrush,0,0,m_nWidth,m_nHeight);
		m_pMemGraphics->DrawImage(pImg,0,0,m_nWidth,m_nHeight);
	}
}


void CImageMixerView::OnDestroy() 
{
	CScrollView::OnDestroy();
	
}

bool CImageMixerView::IsValidImageFile(CString &strFileName)
{
	bool bRet = false;

	do
	{
		int nDotPos = strFileName.ReverseFind('.');
		if(-1==nDotPos)
		{
			break;
		}

		CString strExt = strFileName.Right(strFileName.GetLength()-nDotPos-1);
		if( 0==strExt.CompareNoCase(_T("bmp")) || 
			0==strExt.CompareNoCase(_T("jpg")) || 
			0==strExt.CompareNoCase(_T("gif")) ||
			0==strExt.CompareNoCase(_T("png")) ||
			0==strExt.CompareNoCase(_T("tiff")) )
		{
			bRet = true;
		}
	}
	while(0);

	return bRet;

}

BOOL CImageMixerView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	//return CScrollView::OnEraseBkgnd(pDC);

	return TRUE;
}

void CImageMixerView::AdjustScroll()
{

	//Resize Scrollbars
	CSize sizeTotal;
	// TODO: calculate the total size of this view
	sizeTotal.cx = m_nWidth;
	sizeTotal.cy = m_nHeight;
	SetScrollSizes(MM_TEXT, sizeTotal);


}


void CImageMixerView::OnInsertImage() 
{
	do
	{

		static TCHAR BASED_CODE szFilter[] = _T("Image Files (*.bmp;*.jpg;*.gif;*.png;*.tiff)|*.bmp;*.jpg;*.gif;*.png;*.tiff|All Files (*.*)|*.*||");
 
		CFileDialog FileOpenDlg(TRUE,_T("png"),NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
								szFilter,NULL);
		if(IDOK == FileOpenDlg.DoModal())
		{
			CString strFileName = FileOpenDlg.GetPathName();
			if(!strFileName.IsEmpty())
			{
				//Check for valid image extensions
				if(!IsValidImageFile(strFileName))
				{
					AfxMessageBox(_T("Unsupported Image type!!!"));
					break;
				}

				CImage *pImage = new CImage(strFileName);
				m_vShapeList.push_back(pImage);
				pImage->Draw(*m_pMemGraphics);

				m_pSelectedShape = pImage;
				DeSelectAll();
				pImage->Select();
				m_SettingsDlg.SetTransParency(pImage->GetTransparency());

				RefreshMemDC();

				Invalidate();
			}
		}
	}
	while(0);
}

void CImageMixerView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	bool bFound = false;

	bool bNeedRefresh = false;
			
	SHAPE_VECT::reverse_iterator it;
	for(it = SHAPE_VECT::reverse_iterator(m_vShapeList.end());
	    it!= SHAPE_VECT::reverse_iterator(m_vShapeList.begin());++it)
	{
		CShape* pShape = (CShape*)(*it);

		if(pShape->HitTest(point))
		{
			bFound = true;
			if(!pShape->IsSelected())
			{
				m_pSelectedShape = pShape;
				DeSelectAll();
				pShape->Select();
				m_SettingsDlg.SetTransParency(pShape->GetTransparency());
				bNeedRefresh = true;
			}
			break;
		}
	}

	if(!bFound && m_pSelectedShape)
	{
		m_pSelectedShape = NULL;
		DeSelectAll();
		m_SettingsDlg.SetTransParency(-1);
		bNeedRefresh = true;
	}

	if(bNeedRefresh)
	{
		RefreshMemDC();

		Invalidate();
	}
	
	CScrollView::OnLButtonDown(nFlags, point);
}

void CImageMixerView::DeSelectAll()
{
	SHAPE_VECT::iterator it;
	for(it = m_vShapeList.begin();it!=m_vShapeList.end();++it)
	{
		CShape* pShape = (CShape*)(*it);
		pShape->Select(false);
	}
}

void CImageMixerView::DrawAllShapes()
{
	SHAPE_VECT::iterator it;
	for(it = m_vShapeList.begin();it!=m_vShapeList.end();++it)
	{
		CShape* pShape = (CShape*)(*it);
		pShape->Draw(*m_pMemGraphics);
	}
}



UINT CImageMixerView::OnNcHitTest(CPoint point) 
{
	
	return CScrollView::OnNcHitTest(point);
}

void CImageMixerView::SetTransParency(int nTrans)
{
	if(m_pSelectedShape)
	{
		m_pSelectedShape->SetTransparency(nTrans);

		RefreshMemDC();

		InvalidateRect(m_pSelectedShape->GetBounds());
	}
}

void CImageMixerView::OnFileSave() 
{
	do
	{
		static TCHAR BASED_CODE szFilter[] = _T("Image Files (*.bmp;*.jpg;*.gif;*.png;*.tiff)|*.bmp;*.jpg;*.gif;*.png;*.tiff|All Files (*.*)|*.*||");
 
		CFileDialog FileSaveDlg(FALSE,_T("png"),NULL,OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
								szFilter,NULL);
		if(IDOK == FileSaveDlg.DoModal())
		{
			CString strFileName = FileSaveDlg.GetPathName();
			//Check for valid image extensions
			if(!IsValidImageFile(strFileName))
			{
				AfxMessageBox(_T("Unsupported Image type!!!"));
				break;
			}

			if(!strFileName.IsEmpty())
			{

				int nDotPos = strFileName.ReverseFind('.');
				if(-1==nDotPos)
				{
					break;
				}

				CString strExt = strFileName.Right(strFileName.GetLength()-nDotPos-1);
				CString strEncStr = _T("image/png");
				if(0==strExt.CompareNoCase(_T("bmp")))
				{
					strEncStr = _T("image/bmp");
				}
				else if(0==strExt.CompareNoCase(_T("jpg")))
				{
					strEncStr = _T("image/jpeg");
				}
				else if(0==strExt.CompareNoCase(_T("gif")))
				{
					strEncStr = _T("image/gif");
				}
				else if(0==strExt.CompareNoCase(_T("png")))
				{
					strEncStr = _T("image/png");
				}
				else if(0==strExt.CompareNoCase(_T("tiff")))
				{
					strEncStr = _T("image/tiff");
				}

				CLSID pngClsid;
				GetEncoderClsid(strEncStr, &pngClsid);
				m_pMemBitmap->Save(strFileName, &pngClsid, NULL);
			}
		}
	}
	while(0);
}

int CImageMixerView::GetEncoderClsid(const WCHAR* format, CLSID* pClsid)
{
   UINT  num = 0;          // number of image encoders
   UINT  size = 0;         // size of the image encoder array in bytes

   ImageCodecInfo* pImageCodecInfo = NULL;

   GetImageEncodersSize(&num, &size);
   if(size == 0)
      return -1;  // Failure

   pImageCodecInfo = (ImageCodecInfo*)(malloc(size));
   if(pImageCodecInfo == NULL)
      return -1;  // Failure

   GetImageEncoders(num, size, pImageCodecInfo);

   for(UINT j = 0; j < num; ++j)
   {
      if( wcscmp(pImageCodecInfo[j].MimeType, format) == 0 )
      {
         *pClsid = pImageCodecInfo[j].Clsid;
         free(pImageCodecInfo);
         return j;  // Success
      }    
   }

   free(pImageCodecInfo);
   return -1;  // Failure
}

void CImageMixerView::OnUpdateFileNew(CCmdUI* pCmdUI) 
{
	
	pCmdUI->Enable(FALSE);
}
